@@ -3,7 +3,7 @@ use bevy::{
     render::{RenderApp, RenderStage, RenderWorld},
     winit::WinitWindows,
 };
-use ringbuffer::{RingBufferExt, RingBufferWrite};
+use ringbuffer::{ConstGenericRingBuffer, RingBufferExt, RingBufferWrite};
 use std::time::{Duration, Instant};
 
 #[derive(Debug, Clone, Component)]
@@ -16,6 +16,7 @@ pub struct FramepacePlugin {
     /// likely, but increases motion-to-photon latency of user input rendered to screen. Use
     /// `FramepacePlugin::default()` as a starting point.
     pub safety_margin: Duration,
+    pub power_saver: PowerSaver,
 }
 impl Plugin for FramepacePlugin {
     fn build(&self, app: &mut App) {
@@ -35,12 +36,13 @@ impl Default for FramepacePlugin {
             enabled: true,
             framerate_limit: FramerateLimit::Auto,
             warn_on_frame_drop: true,
-            safety_margin: Duration::from_micros(50),
+            safety_margin: Duration::from_micros(200),
+            power_saver: PowerSaver::Enabled(Duration::from_millis(500)),
         }
     }
 }
 impl FramepacePlugin {
-    pub fn framerate(fps: u64) -> Self {
+    pub fn framerate(fps: u16) -> Self {
         Self {
             framerate_limit: FramerateLimit::Manual(fps),
             ..Default::default()
@@ -57,27 +59,39 @@ pub enum FramerateLimit {
     /// Uses the window's refresh rate to set the framerate limit
     Auto,
     /// Set a manual framerate limit. Note this should be <= to the window's refresh rate.
-    Manual(u64),
+    Manual(u16),
 }
+
+#[derive(Debug, Clone)]
+pub enum PowerSaver {
+    // Disable all power saving features
+    Disabled,
+    /// Sleep the application when it does not have focus, set with the desired frametime, i.e.
+    /// seconds per frame.
+    Enabled(Duration),
+}
+
 #[derive(Debug, Clone, Component)]
-pub struct MeasuredFramerateLimit(u64);
+pub struct MeasuredFramerateLimit(u16);
 
 const FRAMETIME_SAMPLES: usize = 32;
 
 #[derive(Debug)]
 struct FrameTimer {
-    frame_time_history: ringbuffer::ConstGenericRingBuffer<Duration, FRAMETIME_SAMPLES>,
+    frame_time_history: ConstGenericRingBuffer<Duration, FRAMETIME_SAMPLES>,
     post_render_start: Instant,
     render_start: Instant,
     exact_sleep: Duration,
 }
 impl Default for FrameTimer {
     fn default() -> Self {
+        let mut frame_time_history = ConstGenericRingBuffer::default();
+        frame_time_history.fill(Duration::from_millis(1));
         FrameTimer {
-            frame_time_history: Default::default(),
+            frame_time_history,
             post_render_start: Instant::now(),
             render_start: Instant::now(),
-            exact_sleep: Duration::from_millis(0),
+            exact_sleep: Duration::ZERO,
         }
     }
 }
@@ -88,6 +102,12 @@ fn measure_refresh_rate(
     windows: Res<Windows>,
     mut meas_limit: ResMut<MeasuredFramerateLimit>,
 ) {
+    if let PowerSaver::Enabled(t) = settings.power_saver {
+        let window = windows.get_primary().unwrap();
+        if !window.is_focused() {
+            std::thread::sleep(t);
+        }
+    }
     if !settings.is_changed() && !winit.is_changed() {
         return;
     }
@@ -99,7 +119,7 @@ fn measure_refresh_rate(
                 .current_monitor()
                 .unwrap()
                 .video_modes();
-            let best = modes.map(|f| f.refresh_rate() as u64).max();
+            let best = modes.map(|f| f.refresh_rate() as u16).max();
             if let Some(framerate) = best {
                 if framerate != meas_limit.0 {
                     Some(framerate)
@@ -141,16 +161,18 @@ fn framerate_limit_forward_estimator(
 ) {
     let framerate_limit = refresh_rate.0;
     let render_end = Instant::now();
-    let target_frametime = Duration::from_micros(1_000_000 / framerate_limit);
+    let target_frametime = Duration::from_micros(1_000_000 / framerate_limit as u64);
     let last_frametime = render_end.duration_since(timer.post_render_start);
     timer.frame_time_history.push(last_frametime);
+    // let avg_frametime =
+    //     timer.frame_time_history.iter().sum::<Duration>() / FRAMETIME_SAMPLES as u32;
     let max_frametime = timer
         .frame_time_history
         .iter()
         .max()
         .cloned()
         .unwrap_or_else(|| Duration::from_millis(100));
-    let last_render_time = max_frametime - timer.exact_sleep;
+    let last_render_time = max_frametime - max_frametime.min(timer.exact_sleep);
     let estimated_cpu_time_needed = last_render_time + settings.safety_margin;
     let estimated_sleep_time = target_frametime - target_frametime.min(estimated_cpu_time_needed);
     if settings.enabled {
@@ -166,7 +188,7 @@ fn framerate_exact_limiter(
 ) {
     let framerate_limit = refresh_rate.0;
     let system_start = Instant::now();
-    let target_frametime = Duration::from_micros(1_000_000 / framerate_limit);
+    let target_frametime = Duration::from_micros(1_000_000 / framerate_limit as u64);
     let this_frametime = system_start.duration_since(timer.render_start);
     if this_frametime > target_frametime && settings.warn_on_frame_drop {
         warn!(
